package org.processmining.traceclustering.algorithms.affinity.impl;

import org.processmining.traceclustering.algorithms.affinity.AbstractVectorSimilarity;
import org.processmining.traceclustering.models.impl.CaseVector;

/**
 * Cosine vector similarity with equal weights for all dimensions.
 * 
 * @author B.F.A. Hompes <b.f.a.hompes@tue.nl>
 *
 */
public class CosineSimilarity extends AbstractVectorSimilarity {

	public double computeSimilarity(CaseVector firstVector, CaseVector secondVector) {
		// If the vectors are of different sizes we cannot compute cosine similarity.
		if (firstVector.size() != secondVector.size())
			return 0.0;

		// If both vectors are zero vectors, return 1 (exactly similar)
		if (firstVector.isZeroVector() && secondVector.isZeroVector())
			return 1.0;

		// If only one of the vectors are zero vectors, return 0 (cannot compute actually)
		if (firstVector.isZeroVector() || secondVector.isZeroVector())
			return 0.0;

		// Cosine similarity is the dot product divided by the product of the first norms of the two vectors
		// If vectors are huge they might overflow, so then use TraceVector.Norm.TwoRobust. Otherwise use the faster Norm.Two.
		// We might also first normalize the vectors, to weigh their respective perspectives.

		// Initialize variables
		Double dot = 0.0;

		// Compute the dot product
		//		dot = firstVector.dot(secondVector);
		//		return dot / (firstVector.norm(TraceVector.Norm.TwoRobust) * secondVector.norm(TraceVector.Norm.TwoRobust));

		dot = firstVector.toUnitVector().dot(secondVector.toUnitVector());
		return dot / (firstVector.toUnitVector().norm(CaseVector.Norm.TwoRobust)
				* secondVector.toUnitVector().norm(CaseVector.Norm.TwoRobust));
	}

}