package org.processmining.traceclustering.models;

import java.util.HashSet;

import org.deckfour.xes.model.XLog;
import org.processmining.markovclustering.models.MCLMatrix;
import org.processmining.traceclustering.models.impl.CaseCluster;
import org.processmining.traceclustering.parameters.CaseClusteringParameters;

/**
 * A traceclustering is a set of traceclusters. It also holds the eventlog, the
 * traceclustering parameters and the trace similarity matrix.
 * 
 * @author B.F.A. Hompes <b.f.a.hompes@tue.nl>
 *
 */
public abstract class AbstractCaseClustering extends HashSet<CaseCluster> {

	private static final long serialVersionUID = -6358521480293440247L;

	protected XLog eventlog;
	protected CaseClusteringParameters caseClusteringParameters;
	protected MCLMatrix similarityMatrix;

	@Override
	public boolean add(CaseCluster newCluster) {
		AbstractCaseClustering copy = (AbstractCaseClustering) this.clone();
		for (CaseCluster cluster : copy) {
			// check whether the cluster to add is contained in cluster that is already present,
			// if so, do not add the new cluster
			if (cluster.containsAll(newCluster))
				return false;
			// check whether the present cluster is completely contained in the cluster to be added,
			// if so, remove it.
			if (newCluster.containsAll(cluster))
				this.remove(cluster);
		}
		return super.add(newCluster);
	}

	public XLog getEventlog() {
		return eventlog;
	}

	public void setEventlog(XLog eventlog) {
		this.eventlog = eventlog;
	}

	public MCLMatrix getSimilarityMatrix() {
		return similarityMatrix;
	}

	public void setSimilarityMatrix(MCLMatrix similarityMatrix) {
		this.similarityMatrix = similarityMatrix;
	}

	public CaseClusteringParameters getCaseClusteringParameters() {
		return caseClusteringParameters;
	}

	public void setCaseClusteringParameters(CaseClusteringParameters parameters) {
		this.caseClusteringParameters = parameters;
	}

	public void print() {
		System.out.println(this.size() + " clusters found:");
		for (CaseCluster cluster : this) {
			System.out.println("cluster (size: " + cluster.size() + "\t annotation: "
					+ cluster.getAnnotation().toString() + "): ");
			for (Integer trace : cluster) {
				System.out.print(eventlog.get(trace).getAttributes().get("concept:name").toString() + ", ");
			}
			System.out.println();
		}
	}

	@Override
	public boolean equals(Object object) {
		if (!(object instanceof AbstractCaseClustering))
			return false;

		AbstractCaseClustering obj = (AbstractCaseClustering) object;

		if (!eventlog.equals(obj.getEventlog()))
			return false;

		if (!caseClusteringParameters.equals(obj.getCaseClusteringParameters()))
			return false;

		if (!similarityMatrix.equals(obj.getSimilarityMatrix()))
			return false;

		return super.equals(object);
	}
}