package org.processmining.traceclustering.models.impl;

import no.uib.cipr.matrix.DenseVector;

/**
 * Vector for keeping track of age of cases. The value is used in evolutionary
 * trace clustering in order to reduce the effect/weight of older cases.
 * 
 * @author B.F.A. Hompes <b.f.a.hompes@tue.nl>
 *
 */
public class AgeVector extends DenseVector {

	private static final long serialVersionUID = -3415793257469361953L;

	public AgeVector(int size) {
		super(size);
	}

	public AgeVector(AgeVector a) {
		super(a);
	}

	public AgeVector(int size, double defaultValue) {
		super(size);
		for (int i = 0; i < size; i++) {
			set(i, defaultValue);
		}
	}

	@Override
	public void set(int index, double value) {
		assert (value >= 0.0 && value <= 1.0);
		super.set(index, value);
	}

	@Override
	public AgeVector scale(double factor) {
		super.scale(factor);
		return this;
	}

}
