package org.processmining.traceclustering.models.impl;

import java.util.LinkedHashMap;

import org.processmining.traceclustering.models.perspectives.AbstractPerspective;

/**
 * Class that holds a description / annotation for every relevant perspective.
 * 
 * @author B.F.A. Hompes <b.f.a.hompes@tue.nl>
 *
 */
public class CaseClusterAnnotation extends LinkedHashMap<AbstractPerspective, String> {

	private static final long serialVersionUID = -5232899737012459719L;

	@Override
	public boolean equals(Object object) {
		if (!(object instanceof CaseClusterAnnotation))
			return false;

		CaseClusterAnnotation obj = (CaseClusterAnnotation) object;

		// Should have same size
		if (obj.size() != this.size())
			return false;

		// Should have same dimensions
		if (obj.keySet() != this.keySet())
			return false;

		// Should have same values for dimensions
		if (obj.values() != this.values())
			return false;

		// Everything OK, objects are the same
		return true;
	}

	@Override
	public String toString() {
		String result = "";
		for (AbstractPerspective dimension : this.keySet()) {
			result += dimension.getDescription();
			result += ": ";
			result += this.get(dimension);
			result += ", ";
			result += "\t ";
		}
		if (result.contains(","))
			return result.substring(0, result.lastIndexOf(","));
		return result;
	}
}