package org.processmining.traceclustering.models.perspectives;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.traceclustering.models.impl.CaseCluster;
import org.processmining.traceclustering.models.impl.CaseVector;

/**
 * Perspective for the mapping of case to vector. Could be event name alphabet,
 * event name sequence, resource alphabet, maximal repeats etc.
 * 
 * @author B.F.A. Hompes
 *
 */
public abstract class AbstractPerspective {

	private String description;
	private boolean isSelected;

	public AbstractPerspective(String description) {
		setDescription(description);
		setSelected(false);
	}

	public AbstractPerspective(String description, boolean isSelected) {
		setDescription(description);
		setSelected(isSelected);
	}

	public abstract CaseVector projectCase(XLog eventlog, XTrace trace);

	public abstract String createAnnotation(XLog eventlog, CaseCluster cluster);

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public boolean isSelected() {
		return isSelected;
	}

	public void setSelected(boolean isSelected) {
		this.isSelected = isSelected;
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof AbstractPerspective))
			return false;

		AbstractPerspective perspective = (AbstractPerspective) obj;

		if (!description.equals(perspective.getDescription()))
			return false;

		if (isSelected != perspective.isSelected())
			return false;

		return true;
	}

	@Override
	public abstract AbstractPerspective clone();
}