package org.processmining.traceclustering.models.perspectives.impl;

import org.deckfour.xes.classification.XEventAttributeClassifier;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.traceclustering.models.impl.CaseCluster;
import org.processmining.traceclustering.models.impl.CaseVector;
import org.processmining.traceclustering.models.perspectives.AbstractPerspective;

public class EventAlphabetFrequencyPerspective extends AbstractPerspective {

	private XLogInfo logInfo;
	private XEventAttributeClassifier classifier;

	public EventAlphabetFrequencyPerspective(String description, XLogInfo logInfo,
			XEventAttributeClassifier classifier) {
		super(description);
		setClassifier(classifier);
		setLogInfo(logInfo);
	}

	public EventAlphabetFrequencyPerspective(String description, boolean isSelected, XLogInfo logInfo,
			XEventAttributeClassifier classifier) {
		super(description, isSelected);
		setClassifier(classifier);
		setLogInfo(logInfo);
	}

	public XEventAttributeClassifier getClassifier() {
		return classifier;
	}

	public void setClassifier(XEventAttributeClassifier classifier) {
		this.classifier = classifier;
	}

	public XLogInfo getLogInfo() {
		return logInfo;
	}

	private void setLogInfo(XLogInfo logInfo) {
		this.logInfo = logInfo;
	}

	@Override
	public CaseVector projectCase(XLog eventlog, XTrace trace) {
		int differentClasses = logInfo.getEventClasses().getClasses().size();
		CaseVector vector = new CaseVector(differentClasses);

		for (int i = 0; i < differentClasses; i++) {
			XEventClass eventclass = logInfo.getEventClasses().getByIndex(i);
			int count = 0;
			for (XEvent event : trace) {
				if (classifier.getClassIdentity(event).equals(eventclass.getId())) {
					count++;
				}
			}
			vector.set(i, count);
		}
		return vector;
	}

	@Override
	public String createAnnotation(XLog eventlog, CaseCluster cluster) {
		StringBuilder builder = new StringBuilder();

		XLogInfo logInfo = XLogInfoFactory.createLogInfo(eventlog, classifier);

		for (int i = 0; i < logInfo.getEventClasses().getClasses().size(); i++) {
			XEventClass eventclass = logInfo.getEventClasses().getByIndex(i);
			Integer count = 0;
			for (Integer trace : cluster) {
				for (XEvent event : eventlog.get(trace)) {
					if (classifier.getClassIdentity(event).equals(eventclass.getId())) {
						count++;
					}
				}
			}
			if (count > 0) {
				builder.append(eventclass.getId());
				builder.append(": ");
				builder.append(count);
				builder.append(", \t");
			}
		}
		return builder.toString();
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof EventAlphabetFrequencyPerspective))
			return false;

		EventAlphabetFrequencyPerspective perspective = (EventAlphabetFrequencyPerspective) obj;

		if (!classifier.equals(perspective.getClassifier()))
			return false;

		return super.equals(perspective);
	}

	@Override
	public EventAlphabetFrequencyPerspective clone() {
		EventAlphabetFrequencyPerspective perspective = new EventAlphabetFrequencyPerspective(getDescription(),
				isSelected(), getLogInfo(), getClassifier());
		return perspective;
	}
}