package org.processmining.traceclustering.models.perspectives.impl;

import org.deckfour.xes.classification.XEventAttributeClassifier;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.traceclustering.models.impl.CaseCluster;
import org.processmining.traceclustering.models.impl.CaseVector;
import org.processmining.traceclustering.models.perspectives.AbstractPerspective;

public class EventAlphabetOccurrencePerspective extends AbstractPerspective {

	private XEventAttributeClassifier classifier;
	private XLogInfo logInfo;

	public EventAlphabetOccurrencePerspective(String description, XLogInfo logInfo,
			XEventAttributeClassifier classifier) {
		super(description);
		setClassifier(classifier);
		setLogInfo(logInfo);
	}

	public EventAlphabetOccurrencePerspective(String description, boolean isSelected, XLogInfo logInfo,
			XEventAttributeClassifier classifier) {
		super(description, isSelected);
		setClassifier(classifier);
		setLogInfo(logInfo);
	}

	public XEventAttributeClassifier getClassifier() {
		return classifier;
	}

	public void setClassifier(XEventAttributeClassifier classifier) {
		this.classifier = classifier;
	}

	public XLogInfo getLogInfo() {
		return logInfo;
	}

	private void setLogInfo(XLogInfo logInfo) {
		this.logInfo = logInfo;
	}

	@Override
	public CaseVector projectCase(XLog eventlog, XTrace trace) {
		int differentClasses = logInfo.getEventClasses().getClasses().size();
		CaseVector vector = new CaseVector(differentClasses);

		for (int i = 0; i < differentClasses; i++) {
			XEventClass eventclass = logInfo.getEventClasses().getByIndex(i);
			innerloop: for (XEvent event : trace) {
				if (classifier.getClassIdentity(event).equals(eventclass.getId())) {
					vector.set(i, 1.0);
					continue innerloop;
				}
			}
		}
		return vector;
	}

	@Override
	public String createAnnotation(XLog eventlog, CaseCluster cluster) {
		StringBuilder builder = new StringBuilder();

		XLogInfo logInfo = XLogInfoFactory.createLogInfo(eventlog, classifier);

		eventclass: for (int i = 0; i < logInfo.getEventClasses().getClasses().size(); i++) {
			XEventClass eventclass = logInfo.getEventClasses().getByIndex(i);
			for (Integer trace : cluster) {
				for (XEvent event : eventlog.get(trace)) {
					if (classifier.getClassIdentity(event).equals(eventclass.getId())) {
						builder.append(eventclass.getId());
						builder.append(", \t");
						continue eventclass;
					}
				}
			}
		}
		return builder.toString();
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof EventAlphabetOccurrencePerspective))
			return false;

		EventAlphabetOccurrencePerspective perspective = (EventAlphabetOccurrencePerspective) obj;

		if (!classifier.equals(perspective.getClassifier()))
			return false;

		return super.equals(perspective);
	}

	@Override
	public EventAlphabetOccurrencePerspective clone() {
		EventAlphabetOccurrencePerspective perspective = new EventAlphabetOccurrencePerspective(getDescription(),
				isSelected(), getLogInfo(), getClassifier());
		return perspective;
	}
}