package org.processmining.traceclustering.models.perspectives.impl;

import java.util.HashSet;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.traceclustering.models.impl.CaseCluster;
import org.processmining.traceclustering.models.impl.CaseVector;
import org.processmining.traceclustering.models.perspectives.AbstractPerspective;

public class TraceAlphabetOccurrencePerspective extends AbstractPerspective {

	private XLogInfo logInfo;
	private String attributeKey;
	private HashSet<XAttribute> attributeValues;

	public TraceAlphabetOccurrencePerspective(String description, XLogInfo logInfo, String attributeKey,
			HashSet<XAttribute> attributeValues) {
		super(description);
		setLogInfo(logInfo);
		setAttributeKey(attributeKey);
		setAttributeValues(attributeValues);
	}

	public TraceAlphabetOccurrencePerspective(String description, boolean isSelected, XLogInfo logInfo,
			String attributeKey, HashSet<XAttribute> attributeValues) {
		super(description, isSelected);
		setLogInfo(logInfo);
		setAttributeKey(attributeKey);
		setAttributeValues(attributeValues);
	}

	public XLogInfo getLogInfo() {
		return logInfo;
	}

	public void setLogInfo(XLogInfo logInfo) {
		this.logInfo = logInfo;
	}

	public String getAttributeKey() {
		return attributeKey;
	}

	public void setAttributeKey(String attributeKey) {
		this.attributeKey = attributeKey;
	}

	public HashSet<XAttribute> getAttributeValues() {
		return attributeValues;
	}

	public void setAttributeValues(HashSet<XAttribute> attributeValues) {
		this.attributeValues = attributeValues;
	}

	@Override
	public CaseVector projectCase(XLog eventlog, XTrace trace) {
		CaseVector vector = new CaseVector(attributeValues.size());

		int index = 0;
		for (XAttribute value : attributeValues) {
			if (value.equals(trace.getAttributes().get(attributeKey))) {
				vector.set(index, 1.0);
			}
			index++;
		}

		return vector;
	}

	@Override
	public String createAnnotation(XLog eventlog, CaseCluster cluster) {
		StringBuilder builder = new StringBuilder();
		attribute: for (XAttribute value : attributeValues) {
			for (Integer trace : cluster) {
				if (value.equals(eventlog.get(trace).getAttributes().get(attributeKey))) {
					builder.append(value.toString());
					builder.append(", \t");
					continue attribute;
				}
			}
		}
		return builder.toString();
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof TraceAlphabetOccurrencePerspective))
			return false;

		TraceAlphabetOccurrencePerspective perspective = (TraceAlphabetOccurrencePerspective) obj;

		if (!logInfo.equals(perspective.logInfo))
			return false;

		return super.equals(perspective);
	}

	@Override
	public TraceAlphabetOccurrencePerspective clone() {
		TraceAlphabetOccurrencePerspective perspective = new TraceAlphabetOccurrencePerspective(getDescription(),
				isSelected(), getLogInfo(), getAttributeKey(), getAttributeValues());
		return perspective;
	}
}