package org.processmining.traceclustering.parameters;

import java.util.ArrayList;

import org.processmining.basicutils.parameters.impl.PluginParametersImpl;
import org.processmining.markovclustering.parameters.impl.MarkovClusteringParameters;
import org.processmining.traceclustering.models.perspectives.AbstractPerspective;

/**
 * Parameters object used in trace clustering. Holds which perspectives to
 * cluster upon.
 * 
 * @author B.F.A. Hompes <b.f.a.hompes@tue.nl>
 *
 */
public class CaseClusteringParameters extends PluginParametersImpl {

	private static ArrayList<AbstractPerspective> DEFAULTPERSPECTIVES;

	static {
		DEFAULTPERSPECTIVES = new ArrayList<AbstractPerspective>();
	}

	private ArrayList<AbstractPerspective> perspectives;
	private MarkovClusteringParameters mclParameters;

	public CaseClusteringParameters() {
		setPerspectives(new ArrayList<>(DEFAULTPERSPECTIVES));
		setMclParameters(new MarkovClusteringParameters());
	}

	public CaseClusteringParameters(CaseClusteringParameters parameters) {
		setPerspectives(new ArrayList<AbstractPerspective>(parameters.getPerspectives()));
		setMclParameters(new MarkovClusteringParameters(parameters.getMclParameters()));
	}

	public ArrayList<AbstractPerspective> getPerspectives() {
		return perspectives;
	}

	public void setPerspectives(ArrayList<AbstractPerspective> dimensions) {
		this.perspectives = new ArrayList<AbstractPerspective>();
		for (AbstractPerspective dim : dimensions) {
			this.perspectives.add(dim.clone());
		}
	}

	public MarkovClusteringParameters getMclParameters() {
		return mclParameters;
	}

	public void setMclParameters(MarkovClusteringParameters mclParameters) {
		this.mclParameters = mclParameters;
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof CaseClusteringParameters))
			return false;

		CaseClusteringParameters param = (CaseClusteringParameters) obj;

		if (!param.perspectives.equals(this.getPerspectives()))
			return false;

		if (!param.mclParameters.equals(mclParameters))
			return false;

		return true;
	}
}