package org.processmining.traceclustering.plugins;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(
		name = "Change Event Log Classifier to Event Name Classifier",
		parameterLabels = { "Event log" },
		returnLabels = { "Event log" },
		returnTypes = { XLog.class })
public class OpenLogWithEventNameClassifier {

	@UITopiaVariant(
			affiliation = UITopiaVariant.EHV,
			author = "B.F.A. Hompes",
			email = "b.f.a.hompes@student.tue.nl",
			pack = "BartHompes")
	@PluginVariant(
			variantLabel = "Change Event Log Classifier to Event Name Classifier",
			requiredParameterLabels = { 0 })
	public XLog changeDefault(PluginContext context, XLog eventlog) {
		XLog newLog = (XLog) eventlog.clone();

		context.log("Classifiers in original log:");
		for (XEventClassifier classifier : eventlog.getClassifiers()) {
			context.log(classifier.name());
		}

		eventlog.getClassifiers().clear();
		eventlog.getClassifiers().add(new XEventNameClassifier());

		context.log("Classifier in new log:");
		for (XEventClassifier classifier : eventlog.getClassifiers()) {
			context.log(classifier.name());
		}

		return newLog;
	}
}