package org.processmining.traceclustering.test.plugins;

import java.util.HashMap;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(
		name = "TEST - List unique activities",
		parameterLabels = { "Event log" },
		returnLabels = { "Output" },
		returnTypes = { Object.class },
		categories = { PluginCategory.Analytics },
		keywords = "test, event logs",
		help = "List the unique activities per event log in a set of event logs.",
		userAccessible = true,
		handlesCancel = false,
		mostSignificantResult = 1)
public class FindUniqueActivities {
	@UITopiaVariant(
			affiliation = UITopiaVariant.EHV,
			author = "B.F.A. Hompes",
			email = "b.f.a.hompes@tue.nl")
	@PluginVariant(
			variantLabel = "TEST - List unique activities",
			requiredParameterLabels = { 0 },
			help = "Test the unique activities per event log in a set of event logs.")
	public Object doClusteringPublic(PluginContext context, XLog... logs) {
		return test(context, logs);
	}

	private Object test(PluginContext context, XLog... logs) {
		HashMap<String, HashMap<String, Integer>> list = new HashMap<String, HashMap<String, Integer>>();

		int counter = 0;
		for (XLog log : logs) {
			counter++;
			HashMap<String, Integer> activities = new HashMap<String, Integer>();
			for (XTrace trace : log) {
				for (XEvent event : trace) {
					String name = XConceptExtension.instance().extractName(event);
					if (activities.containsKey(name))
						activities.put(name, activities.get(name) + 1);
					else
						activities.put(name, 1);
				}
			}
			list.put(counter + ": Filtered log from " + XConceptExtension.instance().extractName(log) + " ("
					+ log.size() + " cases)", activities);
		}

		for (String log1 : list.keySet()) {
			HashMap<String, Integer> activities1 = new HashMap<String, Integer>(list.get(log1));

			for (String log2 : list.keySet()) {
				if (log1.equals(log2))
					continue;

				HashMap<String, Integer> activities2 = new HashMap<String, Integer>(list.get(log2));

				for (String activity : activities2.keySet()) {
					if (activities1.containsKey(activity)) {
						activities1.remove(activity);
					}
				}
			}

			if (activities1.keySet().size() > 0) {
				System.out.println("Activities unique to " + log1 + "");
				for (String activity : activities1.keySet()) {
					System.out.println(activities1.get(activity) + ":\t" + activity);
				}
			}
		}

		return null;
	}
}