package org.processmining.traceclustering.test.plugins;

import java.util.ArrayList;
import java.util.Collections;

import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginVariant;

import gnu.trove.set.hash.THashSet;

@Plugin(
		name = "TEST - split parts attribute",
		parameterLabels = { "Event log" },
		returnLabels = { "Log" },
		returnTypes = { XLog.class },
		categories = { PluginCategory.Analytics },
		userAccessible = true)
public class SplitTermsTraceAttribute {

	@UITopiaVariant(
			affiliation = UITopiaVariant.EHV,
			author = "B.F.A. Hompes",
			email = "b.f.a.hompes@tue.nl")
	@PluginVariant(
			variantLabel = "TEST - split parts attribute",
			requiredParameterLabels = { 0 })
	public Object splitTerms(UIPluginContext context, XLog eventlog) {

		return apply(context, eventlog);

	}

	private Object apply(UIPluginContext context, XLog eventlog) {
		XLog logCopy = (XLog) eventlog.clone();

		THashSet<String> partsInLog = new THashSet<String>();

		for (XTrace t : logCopy) {
			if (t.getAttributes().containsKey("parts")) {
				String parts = t.getAttributes().get("parts").toString();
				String[] partsSplit = parts.split(",");
				for (String str : partsSplit) {
					partsInLog.add(str);
				}
			}
		}

		ArrayList<String> partsSorted = new ArrayList<String>(partsInLog);
		Collections.sort(partsSorted);

		for (XTrace t : logCopy) {
			for (int i = 0; i < partsSorted.size(); i++) {
				if (t.getAttributes().containsKey("parts")
						&& t.getAttributes().get("parts").toString().contains(partsSorted.get(i)))
					t.getAttributes().put("part-" + partsSorted.get(i), XFactoryRegistry.instance().currentDefault()
							.createAttributeBoolean("part-" + partsSorted.get(i), true, null));
				else
					t.getAttributes().put("part-" + partsSorted.get(i), XFactoryRegistry.instance().currentDefault()
							.createAttributeBoolean("part-" + partsSorted.get(i), false, null));
			}
		}

		return logCopy;
	}

}