package org.processmining.traceclustering.test.plugins;

import java.util.HashMap;
import java.util.HashSet;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.traceclustering.models.impl.CaseCluster;
import org.processmining.traceclustering.models.impl.CaseClustering;

@Plugin(
		name = "TEST - Test trace clustering result",
		parameterLabels = { "Trace Clustering" },
		returnLabels = { "Test result" },
		returnTypes = { Object.class },
		categories = { PluginCategory.Analytics },
		keywords = "test, trace clustering, sequence clustering",
		help = "Test the results of a trace clustering",
		userAccessible = true,
		handlesCancel = false,
		mostSignificantResult = 1)
public class TestClusteringPlugin {

	@UITopiaVariant(
			affiliation = UITopiaVariant.EHV,
			author = "B.F.A. Hompes",
			email = "b.f.a.hompes@tue.nl")
	@PluginVariant(
			variantLabel = "TEST - Test trace clustering result",
			requiredParameterLabels = { 0 },
			help = "Test the results of a trace clustering")
	public Object doClusteringPublic(PluginContext context, CaseClustering clustering) {

		return test(context, clustering);
	}

	@UITopiaVariant(
			affiliation = UITopiaVariant.EHV,
			author = "B.F.A. Hompes",
			email = "b.f.a.hompes@tue.nl")
	@PluginVariant(
			variantLabel = "TEST - Test trace clustering result",
			requiredParameterLabels = { 0 },
			help = "Test the results of a trace clustering")
	public Object doClusteringPublic(PluginContext context, XLog... logs) {

		return test(context, logs);

	}

	private Object test(PluginContext context, CaseClustering clustering) {
		XLog eventlog = clustering.getEventlog();

		double avgPurity = 0.0;

		HashMap<String, Integer> types = new HashMap<String, Integer>();

		for (CaseCluster cluster : clustering) {
			HashSet<String> clusterTypes = new HashSet<String>();
			HashMap<String, Integer> entropyMap = new HashMap<String, Integer>();

			for (Integer traceIndex : cluster) {
				XTrace trace = eventlog.get(traceIndex);
				String type = trace.getAttributes().get("c").toString();

				clusterTypes.add(type);

				if (entropyMap.containsKey(type))
					entropyMap.put(type, entropyMap.get(type) + 1);
				else
					entropyMap.put(type, 1);
			}
			avgPurity += 1.0 / clusterTypes.size();

			Double entropy = 0.0;
			for (String type : entropyMap.keySet()) {
				double fraction = ((double) entropyMap.get(type)) / cluster.size();
				entropy += fraction * Math.log10(fraction) / Math.log10(2);
			}
			System.out.println("Entropy: " + (-entropy));

			for (String type : clusterTypes) {
				if (types.containsKey(type)) {
					types.put(type, types.get(type) + 1);
				} else {
					types.put(type, 1);
				}
			}
		}

		avgPurity /= clustering.size();

		context.log("avg purity: " + avgPurity);
		System.out.println("avg purity: " + avgPurity);

		for (String type : types.keySet()) {
			context.log("split rate: type " + type + ", " + types.get(type));
			System.out.println("split rate: type " + type + ", " + types.get(type));
		}

		System.out.println("Cluster sizes: ");
		for (CaseCluster cluster : clustering) {
			System.out.println(cluster.size());
		}

		return null;
	}

	private Object test(PluginContext context, XLog... logs) {
		double avgPurity = 0.0;

		HashMap<String, Integer> types = new HashMap<String, Integer>();

		for (XLog log : logs) {
			HashSet<String> clusterTypes = new HashSet<String>();
			HashMap<String, Integer> entropyMap = new HashMap<String, Integer>();

			for (XTrace trace : log) {
				String type = trace.getAttributes().get("c").toString();

				clusterTypes.add(type);

				if (entropyMap.containsKey(type))
					entropyMap.put(type, entropyMap.get(type) + 1);
				else
					entropyMap.put(type, 1);
			}
			avgPurity += 1.0 / clusterTypes.size();

			Double entropy = 0.0;
			for (String type : entropyMap.keySet()) {
				double fraction = ((double) entropyMap.get(type)) / log.size();
				entropy += fraction * Math.log10(fraction) / Math.log10(2);
			}
			System.out.println("Entropy: " + (-entropy));

			for (String type : clusterTypes) {
				if (types.containsKey(type)) {
					types.put(type, types.get(type) + 1);
				} else {
					types.put(type, 1);
				}
			}
		}

		avgPurity /= logs.length;

		context.log("avg purity: " + avgPurity);
		System.out.println("avg purity: " + avgPurity);

		for (String type : types.keySet()) {
			context.log("split rate: type " + type + ", " + types.get(type));
			System.out.println("split rate: type " + type + ", " + types.get(type));
		}

		System.out.println("Cluster sizes: ");
		for (XLog log : logs) {
			System.out.println(log.size());
		}

		return null;
	}

}