package org.processmining.traceclustering.visualization;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

/**
 * Renderer class for a perspective in a list, as a label. Basically shows the
 * description of the perspective but keeps all other information present, so we
 * can pass the selected perspectives back to the clustering plugin.
 * 
 * @author B.F.A. Hompes
 *
 */
public class PerspectiveCellRenderer extends JLabel
		implements ListCellRenderer<org.processmining.traceclustering.models.perspectives.AbstractPerspective> {

	private static final long serialVersionUID = 7638657793398902277L;

	public PerspectiveCellRenderer() {
		setOpaque(true);
	}

	public Component getListCellRendererComponent(
			JList<? extends org.processmining.traceclustering.models.perspectives.AbstractPerspective> list,
			org.processmining.traceclustering.models.perspectives.AbstractPerspective perspective, int index,
			boolean isSelected, boolean cellHasFocus) {

		setText(perspective.getDescription());

		if (isSelected) {
			perspective.setSelected(true);
		} else {
			perspective.setSelected(false);
		}

		if (perspective.isSelected()) {
			setBackground(Color.WHITE);
			setForeground(Color.DARK_GRAY);
		} else {
			setBackground(Color.DARK_GRAY);
			setForeground(Color.WHITE);
		}

		return this;
	}

}
