package org.processmining.traceclustering.visualization.transformers;

import org.apache.commons.collections15.Transformer;
import org.processmining.traceclustering.models.impl.CaseCluster;

/**
 * Transformer for the size of vertices in the graph.
 * 
 * @author B.F.A. Hompes <b.f.a.hompes@tue.nl>
 *
 * @param <V>
 */
public class VertexSizeTransformer<V> implements Transformer<V, Integer> {

	private int size;
	private static int minSize = 5;
	private static int maxSize = 125;

	public VertexSizeTransformer(Integer size) {
		this.size = size;
	}

	@Override
	public Integer transform(V v) {
		// If this is a cluster, set size accordingly
		if (v instanceof CaseCluster) {
			return Math.min(Math.max(minSize, ((CaseCluster) v).size()), maxSize);
		}
		return size;
	}
}